@@ -1,10 +1,9 @@
 ;;; emacs-wiki-httpd.el --- Provide support for serving Emacs Wiki pages
 
-;; Copyright (C) 2001, 2002, 2003, 2004 John Wiegley (johnw AT gnu DOT org)
+;; Copyright (C) 2001, 2002, 2003, 2004 John Wiegley
 
 ;; Emacs Lisp Archive Entry
 ;; Filename: emacs-wiki-httpd.el
-;; Version: 2.66
 ;; Keywords: hypermedia
 ;; Author: John Wiegley (johnw AT gnu DOT org)
 ;;         Alex Schroeder (alex AT gnu DOT org)
@@ -27,8 +26,8 @@
 ;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-;; MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -36,8 +35,11 @@
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(require 'emacs-wiki-project)
 (require 'emacs-wiki)
 (require 'httpd)
+(require 'cgi)
+(require 'vc)
 
 (defgroup emacs-wiki-http nil
   "Options controlling the behaviour of the Emacs Wiki HTTP server.
@@ -176,7 +178,7 @@ regexps in `emacs-wiki-editable-pages'."
   (httpd-send-eof))
 
 (defun emacs-wiki-http-reject (title msg &optional annotation)
-  (with-temp-buffer
+  (emacs-wiki-with-temp-buffer
     (insert msg ".\n")
     (if annotation
         (insert annotation "\n"))
@@ -382,7 +384,7 @@ in memory to speed up serving time."
                 (when (/= (file-modes buffer-file-name) oct)
                   (set-file-modes buffer-file-name oct)))
               (kill-buffer (current-buffer)))
-            (with-temp-buffer
+            (emacs-wiki-with-temp-buffer
               (emacs-wiki-file-alist)   ; force re-check
               (insert "<redirect url=\"" page "\" delay=\"3\">")
               (insert "Thank you, your changes have been saved to " page)
@@ -412,14 +414,14 @@ in memory to speed up serving time."
         (if (not (emacs-wiki-editable-p page-name))
             (emacs-wiki-http-reject "Editing not allowed"
                                     "Editing this Wiki page is not allowed")
-          (with-temp-buffer
+          (emacs-wiki-with-temp-buffer
             (emacs-wiki-setup-edit-page page-name)
             ;; this is required because of the : in the name
             (emacs-wiki-http-send-buffer
              (concat "Edit Wiki Page: " page-name))))))
 
      ((string-match "\\`searchwiki\\?get" page)
-      (with-temp-buffer
+      (emacs-wiki-with-temp-buffer
         (insert "<verbatim>" emacs-wiki-http-search-form "</verbatim>")
         (emacs-wiki-http-send-buffer "Search Wiki Pages")))
 
@@ -465,7 +467,7 @@ in memory to speed up serving time."
   "Serve the given PAGE from this emacs-wiki server."
   ;; index.html is really a reference to the main Wiki page
   (if (string= page "index.html")
-      (setq page (concat "wiki?" emacs-wiki-home-page)))
+      (setq page (concat "wiki?" emacs-wiki-default-page)))
 
   ;; handle the actual request
   (let ((vc-follow-symlinks t)
