@@ -1,17 +1,16 @@
 ;;; emacs-wiki.el --- Maintain a local Wiki using Emacs-friendly markup
 
-;; Copyright (C) 2001, 2002, 2003, 2004 John Wiegley (johnw AT gnu DOT org)
-;; Copyright (C) 2003, 2004 Damien Elmes (arch AT repose DOT cx)
-;; Copyright (C) 2004 Mark Triggs (mst AT dishevelled DOT net)
-;; Copyright (C) 2004 Sacha Chua (sacha AT free DOT net DOT ph)
-;; Copyright (C) 2004 Gary Vaughan (gary AT gnu DOT org)
-;; Copyright (C) 2004 Michael Olson (mwolson AT gnu DOT org)
-;; Parts copyright (C) 2004 Yohanes Santoso
-;;     (ysantoso AT dessyku DOT is-a-geek DOT org)
+;; Copyright (C) 2001, 2002, 2003, 2004 John Wiegley
+;; Copyright (C) 2003, 2004 Damien Elmes
+;; Copyright (C) 2004 Mark Triggs
+;; Copyright (C) 2004 Sacha Chua
+;; Copyright (C) 2004 Gary Vaughan
+;; Copyright (C) 2004, 2005 Michael Olson
+;; Copyright (C) 2004 Yohanes Santoso
+;; Copyright (C) 2005 Markus Hoenicka
 
 ;; Emacs Lisp Archive Entry
 ;; Filename: emacs-wiki.el
-;; Version: 2.66
 ;; Keywords: hypermedia
 ;; Author: John Wiegley (johnw AT gnu DOT org)
 ;;         Alex Schroeder (alex AT gnu DOT org)
@@ -34,8 +33,8 @@
 ;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-;; MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -245,12 +244,39 @@
 ;; Wei-Hao Lin (whlin AT cs DOT cmu DOT edu) found a problem with
 ;; parenthesis nesting in `emacs-wiki-walk-directories' and submitted
 ;; a patch.
+;;
+;; Jay Bromley (jbromley AT gmail DOT com) contributed a 2-line patch
+;; to fix a "Saving emacs-wiki-url-protocols: Invalid function: nil."
+;; error in `emacs-wiki-url-protocols'.
+;;
+;; Markus Hoenicka (markus DOT hoenicka AT mhoenicka DOT de) added
+;; backlink functionality to emacs-wiki.
+;;
+;; Frederik Fouvry (fouvry AT CoLi DOT Uni-SB DOT DE) provided a patch
+;; for `emacs-wiki-wiki-link-target' that helps deal with edge cases
+;; in links like "[[#8]]" (10 changed lines).
+;;
+;; Jim Ottaway (j DOT ottaway AT lse DOT ac DOT uk) provided a patch
+;; (2 lines modified) for `emacs-wiki-wiki-visible-name' that causes
+;; attention to be paid to case during matches.
+;;
+;; Andrea Riciputi (ariciputi AT pito DOT com) gave an initial
+;; implementation for tag completion by means of the
+;; `emacs-wiki-insert-tag' function.
+;;
+;; Yamagata Yoriyuki (yoriyuki AT mbg DOT ocn DOT ne DOT jp) helped
+;; fix an interwiki link issue in `emacs-wiki-make-link'.
+;;
+;; Marco Gidde (marco DOT gidde AT tiscali DOT de) made it so that
+;; emacs-wiki-visit-link can recognize buffer names.
 
-(defvar emacs-wiki-version "2.66"
+(defvar emacs-wiki-version "2.70"
   "The version of emacs-wiki currently loaded.")
 
+(require 'compile)
 (require 'derived)
 (require 'font-lock)
+(require 'info)
 
 ;; load pcomplete if it's available
 (load "pcomplete" t t)
@@ -293,30 +319,12 @@ people familiar with Emacs.  See the documentation for the variable
   :group 'emacs-wiki)
 
 (defcustom emacs-wiki-default-page "WelcomePage"
-  "Name of the default page used by \\[emacs-wiki-find-file]."
+  "Name of the default page used by \\[emacs-wiki-find-file].
+This is also used to resolve a link to a project that has no Wiki
+page specified."
   :type 'string
   :group 'emacs-wiki)
 
-(defcustom emacs-wiki-maintainer 
-  (concat "mailto:webmaster@" (system-name))
-  "URL where the maintainer can be reached.
-
-You can alternatively specify a Wiki page name here.  When you
-do, clicking on a non-existing link will take the user to that
-page."
-  :type 'string
-  :group 'emacs-wiki-publish)
-
-(defcustom emacs-wiki-home-page emacs-wiki-default-page
-  "Title of the Wiki Home page."
-  :type 'string
-  :group 'emacs-wiki-publish)
-
-(defcustom emacs-wiki-index-page "WikiIndex"
-  "Title of the Wiki Index page."
-  :type 'string
-  :group 'emacs-wiki-publish)
-
 (defcustom emacs-wiki-interwiki-names
   '(("GnuEmacs" . "http://www.gnu.org/software/emacs/emacs.html")
     ("TheEmacsWiki" .
@@ -356,6 +364,11 @@ this."
                        (choice (string :tag "URL") function)))
   :group 'emacs-wiki)
 
+(defcustom emacs-wiki-create-backlinks nil
+  "When true, create hierarchical backlinks on top of new Wiki pages."
+  :type 'boolean
+  :group 'emacs-wiki)
+
 (defun emacs-wiki-count-chars (string char)
   (let ((i 0)
         (l (length string))
@@ -366,47 +379,13 @@ this."
       (setq i (1+ i)))
     count))
 
-(defun emacs-wiki-set-sym-and-url-regexp (sym value)
-  (when (eq sym 'emacs-wiki-url-protocols)
-    (setq emacs-wiki-url-protocols value)
-    (setq emacs-wiki-url-regexp
-          (concat "\\<\\("
-                  (mapconcat 'car emacs-wiki-url-protocols "\\|")
-                  "\\):"
-                  "[^]["
-                  emacs-wiki-regexp-space
-                  "\"'()<>^`{}]*[^]["
-                  emacs-wiki-regexp-space
-                  "\"'()<>^`{}.,;]+")
-          emacs-wiki-url-server-regexp
-          (concat "\\<\\("
-                  (mapconcat 'car emacs-wiki-url-protocols "\\|")
-                  "\\):"
-                  "\\([^:@]+@[^:]+\\)?"
-                  "\\([^]["
-                  emacs-wiki-regexp-space
-                  "\"'()<>^`{},;/]+\\)"
-                  "\\(/.*\\|$\\)")))
-  (setq emacs-wiki-url-or-name-regexp
-        (concat "\\("
-                (if (eq sym 'emacs-wiki-name-regexp)
-                    value
-                  emacs-wiki-name-regexp) "\\|"
-                  (if (eq sym 'emacs-wiki-name-regexp)
-                      (if (boundp 'emacs-wiki-url-regexp)
-                          emacs-wiki-url-regexp
-                        "")
-                    emacs-wiki-url-regexp) "\\)")
-        emacs-wiki-url-or-name-regexp-group-count
-        (- (emacs-wiki-count-chars
-            emacs-wiki-url-or-name-regexp ?\() 2))
-  (set sym value))
+(provide 'emacs-wiki)
 
-(load "emacs-wiki-regexps" nil t)
+(require 'emacs-wiki-regexps)
 
 (defcustom emacs-wiki-url-protocols
-  '(("info" emacs-wiki-browse-url-info nil)
-    ("man" emacs-wiki-browse-url-man nil)
+  '(("info" emacs-wiki-browse-url-info identity)
+    ("man" emacs-wiki-browse-url-man identity)
     ("google" emacs-wiki-browse-url-google
      emacs-wiki-resolve-url-google)
     ("http" browse-url identity)
@@ -470,16 +449,17 @@ index at intervals."
 Do not rename the wiki page originally referred to."
   (interactive "*")
   (if (emacs-wiki-link-at-point)
-      (let* ((old-link (match-string-no-properties 1))
-             (old-text (match-string-no-properties 3))
+      (let* ((old-link (emacs-wiki-match-string-no-properties 1))
+             (old-text (emacs-wiki-match-string-no-properties 3))
              (match-start (match-beginning 0))
              (match-end (match-end 0))
              (link (emacs-wiki-make-link
                     (read-string "Link: "
                                  (emacs-wiki-link-unescape old-link))
-                    (read-string
-                     "Text: "
-                     (emacs-wiki-link-unescape old-text t)))))
+                    (when old-text
+                      (read-string
+                       "Text: "
+                       (emacs-wiki-link-unescape old-text t))))))
         (delete-region match-start match-end)
         (insert link))
     (error "There is no valid link at point")))
@@ -491,6 +471,8 @@ Do not rename the wiki page originally referred to."
     (define-key map [(control ?c) (control ?b)] 'emacs-wiki-backlink)
     (define-key map [(control ?c) (control ?s)] 'emacs-wiki-search)
     (define-key map [(control ?c) (control ?p)] 'emacs-wiki-publish)
+    (define-key map [(control ?c) (control ?t)]
+      'emacs-wiki-publish-this-page)
     (define-key map [(control ?c) (control ?v)]
       'emacs-wiki-change-project)
     (define-key map [(control ?c) (control ?e)]
@@ -510,6 +492,10 @@ Do not rename the wiki page originally referred to."
     (define-key map [tab] 'emacs-wiki-next-reference)
     (define-key map [(control ?i)] 'emacs-wiki-next-reference)
 
+    (define-key map [(control ?c) tab] 'emacs-wiki-insert-tag)
+    (define-key map [(control ?c) (control ?i)]
+      'emacs-wiki-insert-tag)
+
     (define-key map [(shift tab)] 'emacs-wiki-previous-reference)
     (unless (featurep 'xemacs)
       (define-key map [(shift iso-lefttab)]
@@ -550,6 +536,8 @@ Do not rename the wiki page originally referred to."
 
 (defvar emacs-wiki-project nil)
 
+(require 'emacs-wiki-publish)
+
 ;;;###autoload
 (define-derived-mode emacs-wiki-mode text-mode "Wiki"
   "An Emacs mode for maintaining a local Wiki database.
@@ -598,39 +586,67 @@ All wikis reside in the `emacs-wiki-directories'.
     (set (make-variable-buffer-local
           'pcomplete-parse-arguments-function)
          'emacs-wiki-current-word))
+  ;; Make fill not split up links
+  (when (boundp 'fill-nobreak-predicate)
+    (make-local-variable 'fill-nobreak-predicate)
+    ;; Work around annoying inconsistency in fill handling
+    (if (not (emacs-wiki-extreg-usable-p))
+        (setq fill-nobreak-predicate 'emacs-wiki-fill-nobreak-p)
+      (add-to-list 'fill-nobreak-predicate
+                   'emacs-wiki-fill-nobreak-p)))
+  ;; Make adaptive fill work nicely with item lists
+  (set (make-local-variable 'adaptive-fill-regexp)
+       (concat "[" emacs-wiki-regexp-blank "]*\\(-+["
+               emacs-wiki-regexp-blank
+               "]*\\|[0-9]+\\.["
+               emacs-wiki-regexp-blank "]*\\)*"))
   ;; avoid problems caused by emacs-wiki-url-protocols not getting
   ;; custom-set properly.
   (emacs-wiki-set-sym-and-url-regexp 'emacs-wiki-url-protocols
                                      emacs-wiki-url-protocols)
+  ;; update the project settings -- this used to not be necessary, but
+  ;; apparently the `customize' handling changed slightly in Emacs.
+  (run-hooks 'emacs-wiki-update-project-hook)
   ;; avoid problems caused by the 'intangible' text property with
   ;; paragraph filling.
   (set (make-local-variable 'inhibit-point-motion-hooks) t))
 
+;; Make flyspell not mess with links
+(put 'emacs-wiki-mode
+     'flyspell-mode-predicate
+     'emacs-wiki-flyspell-p)
+
 (defsubst emacs-wiki-page-file (page &optional no-check-p)
   "Return a filename if PAGE exists within the current Wiki."
   (cdr (assoc page (emacs-wiki-file-alist no-check-p))))
 
 ;;; Compatibility functions
 
-(unless (fboundp 'line-end-position)
-  (defsubst line-end-position (&optional N)
+(defun emacs-wiki-line-end-position (&optional N)
+  (if (fboundp 'line-end-position)
+      (line-end-position N)
     (save-excursion (end-of-line N) (point))))
 
-(unless (fboundp 'line-beginning-position)
-  (defsubst line-beginning-position (&optional N)
+(defun emacs-wiki-line-beginning-position (&optional N)
+  (if (fboundp 'line-beginning-position)
+      (line-beginning-position N)
     (save-excursion (beginning-of-line N) (point))))
 
-(unless (fboundp 'match-string-no-properties)
-  (defalias 'match-string-no-properties 'match-string))
+(defun emacs-wiki-match-string-no-properties (num &optional string)
+  "Return string of text matched by last search, without text properties.
+num specifies which parenthesized expression in the last regexp."
+  (if (fboundp 'match-string-no-properties)
+      (match-string-no-properties num string)
+    (match-string num string)))
 
-(if (fboundp 'derived-mode-p)
-    (defalias 'emacs-wiki-derived-mode-p 'derived-mode-p)
-  ;; PUBLIC: find if the current mode derives from another.
-  ;; Taken from GNU Emacs 21 subr.el
-  (defun emacs-wiki-derived-mode-p (&rest modes)
+(defun emacs-wiki-derived-mode-p (&rest modes)
     "Non-nil if the current major mode is derived from one of MODES.
 Uses the `derived-mode-parent' property of the symbol to trace
 backwards."
+  (if (fboundp 'derived-mode-p)
+      (derived-mode-p modes)
+    ;; PUBLIC: find if the current mode derives from another.
+    ;; Taken from GNU Emacs 21 subr.el
     (let ((parent major-mode))
       (while (and (not (memq parent modes))
                   (setq parent (get parent 'derived-mode-parent))))
@@ -657,29 +673,49 @@ If SUBSTITUTE is non-nil, use `replace-match'-style substitutions."
 
 ;;; Other functions
 
+(defun emacs-wiki-fill-nobreak-p ()
+  "Return nil if we should allow a fill to occur at point.
+Otherwise return non-nil.
+
+This is used to keep long extended links from being mangled by
+autofill."
+  (save-excursion
+    (save-match-data
+      (and (re-search-backward "\\[\\[\\|\\]\\]"
+                               (emacs-wiki-line-beginning-position) t)
+           (string= (or (match-string 0) "")
+                    "[[")))))
+
+(defun emacs-wiki-flyspell-p ()
+  "Return non-nil if we should allow spell-checking to occur at point.
+Otherwise return nil.
+
+This is used to keep links from being improperly colorized by flyspell."
+  (save-match-data
+    (null (emacs-wiki-link-at-point))))
+
 (defun emacs-wiki-directories-member (&optional directories)
   "Return non-nil if the current buffer is in `emacs-wiki-directories'."
-  (unless (null buffer-file-name)
-    (let ((here (file-truename          ; resolve symbolic links
-                 (expand-file-name
-                  (file-name-directory buffer-file-name))))
-          (d (or directories emacs-wiki-directories))
-          yes)
-      (while d
-        (let ((dir (file-name-as-directory (if (consp (car d))
-                                               (caar d) (car d)))))
-          (if (and (string-match
-                    (concat "^" (regexp-quote
-                                 (file-truename
-                                  (expand-file-name dir))))
-                    here)
-                   ;; not an ignored file
-                   (not (string-match
-                         emacs-wiki-file-ignore-regexp
-                         buffer-file-name)))
-              (setq yes (car d) d nil)
-            (setq d (cdr d)))))
-      yes)))
+  (let ((name (or emacs-wiki-current-file buffer-file-name)))
+    (unless (null name)
+      (let ((here (file-truename          ; resolve symbolic links
+                   (expand-file-name (file-name-directory name))))
+            (d (or directories emacs-wiki-directories))
+            yes)
+        (save-match-data
+          (while d
+            (let ((dir (file-name-as-directory (if (consp (car d))
+                                                   (caar d) (car d)))))
+              (if (and (string-match
+                        (concat "^" (regexp-quote
+                                     (file-truename
+                                      (expand-file-name dir))))
+                        here)
+                       ;; not an ignored file
+                       (not (string-match emacs-wiki-file-ignore-regexp name)))
+                  (setq yes (car d) d nil)
+                (setq d (cdr d))))))
+        yes))))
 
 (defun emacs-wiki-maybe (&optional check-only)
   "Maybe turn emacs-wiki mode on for this file."
@@ -740,7 +776,9 @@ If SUBSTITUTE is non-nil, use `replace-match'-style substitutions."
 All this means is that certain extensions, like .gz, are removed."
   (save-match-data
     (unless name
-      (setq name buffer-file-name))
+      (setq name (or emacs-wiki-current-file
+                     buffer-file-name
+                     (concat default-directory (buffer-name)))))
     (if name
         (let ((page (file-name-nondirectory name)))
           (if (string-match emacs-wiki-ignored-extensions-regexp page)
@@ -757,7 +795,8 @@ All this means is that certain extensions, like .gz, are removed."
   "Return this page's publishing date."
   (or emacs-wiki-current-page-date
       (format-time-string
-       "%e %B %Y" (nth 5 (file-attributes buffer-file-name)))))
+       "%e %B %Y" (nth 5 (file-attributes (or emacs-wiki-current-file
+                                              buffer-file-name))))))
 
 (defcustom emacs-wiki-recurse-directories
   nil
@@ -798,9 +837,20 @@ users."
 
 (defcustom emacs-wiki-refresh-file-alist-p
   nil
-  "Function that returns non-nil if the file list needs to be refreshed.
-If nil, never refresh. If t, always refresh. See
-`emacs-wiki-refresh-file-alist-maybe'."
+"Decide whether the file list cache needs to be refreshed.
+If this is a function, the return value decides if the file list
+cache needs to be refreshed.
+If a variable, the value nil means never refresh. If non-nil,
+always refresh.
+
+Normally emacs-wiki will try to keep the file name cache up to date,
+but if wiki files are created outside of emacs-wiki you may need to
+experiment with this variable.
+
+Using non-nil for this option will slow down updating of pages
+considerably.
+
+See `emacs-wiki-refresh-file-alist-maybe'."
   :type '(choice
           (const :tag "Do not automatically refresh" nil)
           (const :tag "Always automatically refresh" t)
@@ -836,14 +886,12 @@ Suitable for use as `emacs-wiki-refresh-file-alist-p'."
     ;; `emacs-wiki-current-project' assoc key in
     ;; `emacs-wiki-file-alist' to timestamp this recalculation of the
     ;; list of files.
-    (mapc
-     (lambda (dir)
-       (let ((mod-time (nth 5 (file-attributes dir))))
-         (if (or (null last-mod)
-                 (and mod-time (emacs-wiki-time-less-p last-mod
-                                                       mod-time)))
-             (setq last-mod mod-time))))
-     emacs-wiki-directories)
+    (dolist (dir emacs-wiki-directories)
+      (let ((mod-time (nth 5 (file-attributes dir))))
+        (if (or (null last-mod)
+                (and mod-time (emacs-wiki-time-less-p last-mod
+                                                      mod-time)))
+            (setq last-mod mod-time))))
     (if file-alist
         (setcdr (cdr file-alist) last-mod)
       (setq file-alist (cons emacs-wiki-current-project
@@ -917,12 +965,45 @@ Suitable for use as `emacs-wiki-refresh-file-alist-p'."
       (setq projs (cdr projs)))
     (emacs-wiki-file-alist)))
 
+(defmacro emacs-wiki-with-temp-buffer (&rest body)
+  "Create a temporary buffer, and evaluate BODY there like `progn'.
+See also `with-temp-file' and `with-output-to-string'.
+Unlike `with-temp-buffer', this will never attempt to save the temp buffer."
+  (let ((temp-buffer (make-symbol "temp-buffer")))
+    `(let ((,temp-buffer (generate-new-buffer " *emacs-wiki-temp*")))
+       (unwind-protect
+           (if debug-on-error
+               (with-current-buffer ,temp-buffer
+                 ,@body)
+             (condition-case err
+                 (with-current-buffer ,temp-buffer
+                   ,@body)
+               (error
+                (if (fboundp 'display-warning)
+                    (display-warning 'emacs-wiki
+                                     (format "%s: Error occurred: %s"
+                                             (emacs-wiki-page-name)
+                                             err)
+                                     (if (featurep 'xemacs)
+                                         'warning
+                                       :warning))
+                  (message "%s: Error occurred: %s"
+                           (emacs-wiki-page-name)
+                           err)))))
+         (when (buffer-live-p ,temp-buffer)
+           (with-current-buffer ,temp-buffer
+             (set-buffer-modified-p nil))
+           (unless debug-on-error (kill-buffer ,temp-buffer)))))))
+(put 'emacs-wiki-with-temp-buffer 'lisp-indent-function 0)
+(put 'emacs-wiki-with-temp-buffer 'edebug-form-spec '(body))
+
 ;;; Utility functions to extract parts of a Wiki name
 
 (defsubst emacs-wiki-wiki-url-p (name)
   "Return non-nil if NAME is a URL."
-  (save-match-data
-    (string-match emacs-wiki-url-regexp name)))
+  (and name
+       (save-match-data
+         (string-match emacs-wiki-url-regexp name))))
 
 (defun emacs-wiki-wiki-visible-name-1 (base tag)
   "Figure out what should be displayed for a Wiki link.
@@ -935,8 +1016,9 @@ Either one of BASE or TAG must be specified."
     (concat (if (and emacs-wiki-project-remove-last-word
                      (assoc base emacs-wiki-projects))
                 (replace-regexp-in-string
-                 "[A-Z][a-z]+$" ""
-                 base)
+                 (concat "[" emacs-wiki-regexp-upper "]"
+                         "[" emacs-wiki-regexp-lower "]+$")
+                 "" base)
               base)
             (unless (or (null tag)
                         (string= "" tag))
@@ -946,14 +1028,15 @@ Either one of BASE or TAG must be specified."
   "Return the visible part of a Wiki link.
 This only really means something if [[extended][links]] are involved."
   (save-match-data
-    (let ((name wiki-name))
+    (let ((case-fold-search nil)
+          (name wiki-name))
       (if (string-match emacs-wiki-extended-link-regexp name)
           (if (match-string 2 name)
               (setq name (match-string 3 name))
             (setq name (match-string 1 name))
             (when (string-match "^mailto:" name)
               (setq name (substring name (match-end 0))))))
-      (when (string-match (concat "^" emacs-wiki-name-regexp "$")
+      (when (string-match (concat "^\\(" emacs-wiki-name-regexp "\\)$")
                           name)
         (string-match "#" name)
         (let ((base (substring name 0 (match-beginning 0)))
@@ -1013,7 +1096,8 @@ indicates link destination, unescape brackets and spaces."
   (let ((case-fold-search nil))
     (unless name
       (setq name (emacs-wiki-wiki-visible-name link)))
-    (setq link (emacs-wiki-wiki-link-target link))
+    (when (string-match "^\\[\\[\\([^]]+\\)\\]" link)
+      (setq link (match-string 1 link)))
     (when (equal name link)
       (setq name nil))
     (if name
@@ -1031,9 +1115,6 @@ indicates link destination, unescape brackets and spaces."
           link
         (concat "[[" (emacs-wiki-link-escape link) "]]")))))
 
-(defvar emacs-wiki-bare-digits-anchor-prefix "note"
-  "Prefix to add to anchors composed only of digits.")
-
 (defun emacs-wiki-wiki-link-target (wiki-name)
   "Return the target of a Wiki link.
 This might include anchor tags."
@@ -1041,31 +1122,21 @@ This might include anchor tags."
     (let ((name wiki-name) lookup)
       (if (string-match "^\\[\\[\\([^]]+\\)\\]" name)
           (setq name (match-string 1 name)))
-      (unless (emacs-wiki-page-file name)
-        (string-match "\\`\\([^#]+\\)\\(#\\(.+\\)\\)?\\'" name)
+      (unless (or (not (> (length name) 0))
+                  (emacs-wiki-page-file name))
         (if (and emacs-wiki-interwiki-names
-                 (> (length name) 0)
+                 (string-match "\\`\\([^#]+\\)\\(#\\(.+\\)\\)?\\'" name)
                  (setq lookup (assoc
-                               (if (match-string 1 name)
-                                   (match-string 1 name)
-                                 name)
-                               emacs-wiki-interwiki-names)))
+                                (or (match-string 1 name)
+                                    name)
+                                emacs-wiki-interwiki-names)))
             (let ((tag (match-string 3 name))
                   (target (cdr lookup)))
-              (if (stringp target)
-                  (setq name (concat target tag))
-                (setq name (funcall target tag))))
-          (if (and (> (length name) 0)
-                   (eq (aref name 0) ?#))
-              (setq name (concat (emacs-wiki-page-name) name)))
-          ;; Resolve numeric anchors to
-          ;; `emacs-wiki-bare-digits-anchor-prefix'
-          (when (string-match "#\\([0-9]+\\)" name)
-            (setq name (replace-match
-                        (concat "#"
-                                emacs-wiki-bare-digits-anchor-prefix
-                                (match-string 1 name))
-                        t t name)))))
+              (setq name (if (stringp target)
+                             (concat target tag)
+                           (funcall target tag))))
+          (if (eq (aref name 0) ?#)
+              (setq name (concat (emacs-wiki-page-name) name)))))
       (emacs-wiki-link-unescape name))))
 
 (defun emacs-wiki-wiki-base (wiki-name)
@@ -1144,14 +1215,15 @@ page will be created if it does not already exist."
     (save-excursion
       (if (emacs-wiki-link-at-point)
           (goto-char (match-end 0)))
-      (while (< cycled 2)
-        (if (re-search-forward emacs-wiki-url-or-name-regexp nil t)
-            (when (get-text-property (match-beginning 0)
-                                     'keymap)
-              (setq pos (match-beginning 0)
-                    cycled 2))
-          (goto-char (point-min))
-          (setq cycled (1+ cycled)))))
+      (save-match-data
+        (while (< cycled 2)
+          (if (re-search-forward emacs-wiki-url-or-name-regexp nil t)
+              (when (get-text-property (match-beginning 0)
+                                       'keymap)
+                (setq pos (match-beginning 0)
+                      cycled 2))
+            (goto-char (point-min))
+            (setq cycled (1+ cycled))))))
     (if pos
         (goto-char pos))))
 
@@ -1162,14 +1234,15 @@ This function is not entirely accurate, but it's close enough."
   (let ((case-fold-search nil)
         (cycled 0) pos)
     (save-excursion
-      (while (< cycled 2)
-        (if (re-search-backward emacs-wiki-url-or-name-regexp nil t)
-            (when (get-text-property (match-beginning 0)
-                                     'keymap)
-              (setq pos (point)
-                    cycled 2))
-          (goto-char (point-max))
-          (setq cycled (1+ cycled)))))
+      (save-match-data
+        (while (< cycled 2)
+          (if (re-search-backward emacs-wiki-url-or-name-regexp nil t)
+              (when (get-text-property (match-beginning 0)
+                                       'keymap)
+                (setq pos (point)
+                      cycled 2))
+            (goto-char (point-max))
+            (setq cycled (1+ cycled))))))
     (if pos
         (goto-char pos))))
 
@@ -1185,45 +1258,92 @@ If OTHER-WINDOW is non-nil, open in a different window."
       (when entry
         (funcall (cadr entry) url)))))
 
+(defun emacs-wiki-get-current-backlink ()
+  "Return the backlink in the current page or nil if there is none."
+  (save-excursion
+    (save-match-data
+      (goto-char (point-min))
+      (when (looking-at "\\*\\*/.*\\*\\*\n")
+        (buffer-substring
+         (+ (point-min) 2) (- (match-end 0) 3))))))
+
+(defun emacs-wiki-insert-backlink (backlink &optional parent-links)
+  "Insert BACKLINK in the current page.
+If PARENT-LINKS is non-nil, insert it before BACKLINK."
+  (save-excursion
+    (goto-char (point-min))
+    (insert "**")
+    (when parent-links
+      (insert parent-links " "))
+    (insert "/ " backlink "**\n\n")))
+
 (defun emacs-wiki-visit-link
   (link-name &optional refresh-buffer other-window)
   "Visit the URL or link named by LINK-NAME.
 REFRESH-BUFFER is an optional buffer to refresh on saving the
 visited page.  This makes the bad link face in the linking buffer
 go away."
-  (let ((link (emacs-wiki-wiki-link-target link-name))
-        newbuf)
+  (let ((link (emacs-wiki-wiki-link-target link-name)))
     (if (emacs-wiki-wiki-url-p link)
         (emacs-wiki-browse-url link other-window)
       ;; The name list is current since the last time the buffer was
       ;; highlighted
       (let* ((base (emacs-wiki-wiki-base link-name))
+             (base-buffer (get-buffer base))
              (file (emacs-wiki-page-file base))
              (tag  (and (not (emacs-wiki-wiki-url-p link))
                         (emacs-wiki-wiki-tag link)))
              (find-file-function (if other-window
                                      'find-file-other-window
-                                   'find-file)))
-        (setq newbuf
-              (if (null file)
-                  (funcall find-file-function
-                           (expand-file-name
-                            base
-                            (file-name-directory (buffer-file-name))))
-                (funcall find-file-function file)))
+                                   'find-file))
+             ;; the name of the buffer that contains the link.  check
+             ;; whether buffer-name is a WikiName, else make it one
+             (parent-name (or (emacs-wiki-page-name)
+                              (concat "[[" (buffer-name) "]]")))
+             (parent-allowed-backlink-p
+              (not (string-match
+                    emacs-wiki-exclude-backlink-parent-regexp
+                    parent-name)))
+             (parent-backlink (and parent-allowed-backlink-p
+                                   (emacs-wiki-get-current-backlink)))
+             (newbuf (if (and base-buffer (not (buffer-file-name base-buffer)))
+                         (switch-to-buffer base-buffer)
+                       (if (null file)
+                           (funcall find-file-function
+                                    (expand-file-name
+                                     base
+                                     (file-name-directory
+                                      (buffer-file-name))))
+                         (funcall find-file-function file))))
+             (emacs-wiki-create-backlinks
+              (and emacs-wiki-create-backlinks parent-allowed-backlink-p))
+             (make-link-p (and emacs-wiki-create-backlinks
+                               ;; insert backlink only in new files
+                               (null file)
+                               (not (string-match
+                                     emacs-wiki-exclude-backlink-regexp
+                                     (emacs-wiki-page-name
+                                      (buffer-file-name newbuf)))))))
         (when tag
           (goto-char (point-min))
-          (unless (re-search-forward (concat "^\\.?#" tag) nil t)
-            (when (string-match
-                   (concat emacs-wiki-bare-digits-anchor-prefix
-                           "\\(.+\\)")
-                   tag)
-              (re-search-forward (concat "^\\.?#" (match-string 1 tag)
-                                         "\\>")))))
+          (save-match-data (re-search-forward (concat "^\\.?#" tag) nil t)))
         (when refresh-buffer
+          (when make-link-p
+            (emacs-wiki-insert-backlink parent-name parent-backlink))
+          ;; todo: is with-current-buffer necessary here?
           (with-current-buffer newbuf
             (add-hook 'after-save-hook
-                      'emacs-wiki-refresh-buffers-once t t)))))))
+                      'emacs-wiki-refresh-buffers-once t t)
+            ;; save the new file to avoid another backlink if the
+            ;; buffer is visited again before being saved
+            (when (or make-link-p
+                      (and (null file)
+                           (not parent-allowed-backlink-p)))
+              ;; set modified status to t: pages with excluded
+              ;; parents may not be modified, but we want to save
+              ;; anyway
+              (set-buffer-modified-p t)
+              (save-buffer))))))))
 
 (defun emacs-wiki-refresh-buffers-once ()
   "Refresh all the buffers once from after-save-hook.
@@ -1238,11 +1358,10 @@ Call after creating a page."
   (interactive)
   (emacs-wiki-refresh-file-alist)
   (let ((my-project emacs-wiki-current-project))
-    (mapc (lambda (buffer)
-            (with-current-buffer buffer
-              (when (equal emacs-wiki-current-project my-project)
-                (emacs-wiki-highlight-buffer))))
-          (buffer-list))))
+    (dolist (buffer (buffer-list))
+      (with-current-buffer buffer
+        (when (equal emacs-wiki-current-project my-project)
+          (emacs-wiki-highlight-buffer))))))
 
 (defun emacs-wiki-link-at-point (&optional pos)
   "Return non-nil if a URL or Wiki link name is at point."
@@ -1254,14 +1373,22 @@ Call after creating a page."
             there)
         (save-excursion
           (goto-char here)
-          (skip-chars-backward (concat "^['\"<>{}("
-                                       emacs-wiki-regexp-space))
+          ;; There is a bug in `skip-chars-backward' that makes it
+          ;; necessary to use (concat "\n" emacs-wiki-regexp-blank)
+          ;; rather than just `emacs-wiki-regexp-space'.
+          (skip-chars-backward (concat "^['\"<>{}(\n"
+                                       emacs-wiki-regexp-blank))
           (setq there (point))
-          (or (and (search-backward "[[" (line-beginning-position) t)
-                   (looking-at emacs-wiki-name-regexp)
+          (if (save-match-data
+                (and (re-search-backward "\\[\\[\\|\\]\\]"
+                                         (emacs-wiki-line-beginning-position)
+                                         t)
+                     (string= (or (match-string 0) "")
+                              "[[")))
+              (and (looking-at emacs-wiki-extended-link-regexp)
                    (<= here (match-end 0)))
-              (and (goto-char there)
-                   (looking-at emacs-wiki-url-or-name-regexp)))))))
+            (goto-char there)
+            (looking-at emacs-wiki-url-or-name-regexp))))))
 
 (defun emacs-wiki-follow-name-at-point (&optional other-window)
   "Visit the link at point, or insert a newline if none."
@@ -1274,7 +1401,7 @@ Call after creating a page."
               'emacs-wiki-bad-link-face)
       (setq buf (current-buffer)))
     (if (emacs-wiki-link-at-point)
-        (emacs-wiki-visit-link (match-string-no-properties 0)
+        (emacs-wiki-visit-link (emacs-wiki-match-string-no-properties 0)
                                buf other-window)
       (error "There is no valid link at point"))))
 
@@ -1286,7 +1413,11 @@ Call after creating a page."
 (defun emacs-wiki-follow-name-at-mouse (event &optional other-window)
   "Visit the link at point, or yank text if none."
   (interactive "eN")
-  (let (path)
+  ;; if we're visiting a bad link, pass the current buffer to the
+  ;; visiting function so that it can be refreshed on saving the new
+  ;; page
+  (let ((path)
+        (buf))
     (save-excursion
       (cond ((featurep 'xemacs)    ; XEmacs
              (set-buffer (window-buffer (event-window event)))
@@ -1297,9 +1428,12 @@ Call after creating a page."
                           (posn-window (event-start event))))
              (goto-char (posn-point (event-start event)))))
       (when (emacs-wiki-link-at-point)
-        (setq path (match-string 0))))
+        (setq path (match-string 0))
+        (when (eq (get-text-property (point) 'face)
+                  'emacs-wiki-bad-link-face)
+          (setq buf (current-buffer)))))
     (when path
-      (emacs-wiki-visit-link path nil other-window))))
+      (emacs-wiki-visit-link path buf other-window))))
 
 (defun emacs-wiki-follow-name-at-mouse-other-window (event)
   "Visit the link at point"
@@ -1346,7 +1480,7 @@ link accordingly."
     (if (emacs-wiki-link-at-point)
         (progn
           (setq old-name (emacs-wiki-wiki-base
-                          (match-string-no-properties 0)))
+                          (emacs-wiki-match-string-no-properties 0)))
           (setq new-name
                 (read-from-minibuffer "Rename file to: " old-name))
           (when (string= old-name new-name)
@@ -1440,17 +1574,14 @@ Otherwise the entire wiki file is scanned for a #title.")
 (defun emacs-wiki-get-title-fast (filename)
   "Return the title in FILENAME.
 Scan only the first `emacs-wiki-index-title-threshold' bytes."
-  (with-temp-buffer
-    (insert-file-contents
-     filename
-     nil
-     0
-     (and (integerp emacs-wiki-index-title-threshold)
-          (> emacs-wiki-index-title-threshold 0)
-          emacs-wiki-index-title-threshold))
+  (emacs-wiki-with-temp-buffer
+    (insert-file-contents filename nil 0
+                          (and (integerp emacs-wiki-index-title-threshold)
+                               (> emacs-wiki-index-title-threshold 0)
+                               emacs-wiki-index-title-threshold))
     (goto-char (point-max))
     (when (re-search-backward "^#title\\s-+\\(.+\\)$" nil t)
-      (match-string-no-properties 1))))
+      (emacs-wiki-match-string-no-properties 1))))
 
 (defun emacs-wiki-file-alist-with-titles (&optional verbose)
   "Return a list of pages with titles and filenames.
@@ -1509,7 +1640,7 @@ messages are printed."
                        (emacs-wiki-private-p
                         (emacs-wiki-index-file-page (car files))))
             (insert
-             (if as-list "- " "")
+             (if as-list " - " "")
              (emacs-wiki-make-link
               (emacs-wiki-index-file-page (car files))
               (emacs-wiki-index-file-title (car files)))
@@ -1528,12 +1659,6 @@ messages are printed."
       (emacs-wiki-mode))
     (message "Generating Wiki index...done")))
 
-;; Load extensions
-(load "emacs-wiki-colors" nil t)
-(load "emacs-wiki-macros" nil t)
-(load "emacs-wiki-project" nil t)
-(load "emacs-wiki-publish" nil t)
-
 ;; URLs
 
 (defun emacs-wiki-resolve-url-google (url)
@@ -1571,5 +1696,56 @@ messages are printed."
         ((string-match "^man://\\(.+\\)" url)
          (manual-entry (concat (match-string 1 url))))))
 
-(provide 'emacs-wiki)
+(defvar emacs-wiki-tag-history nil
+  "List of recently-entered tags; used by `emacs-wiki-insert-tag'.
+If you want a tag to start as the default, you may manually set
+this variable to a list.")
+
+(defvar emacs-wiki-custom-tags nil
+  "Keep track of any new tags entered in `emacs-wiki-insert-tag'.
+If there are (X)HTML tags that you use frequently with that
+function, you might want to set this manually.")
+
+(defun emacs-wiki-insert-tag ()
+  "Insert a tag interactively with a blank line after it."
+  (interactive)
+  (let ((markup-tag (completing-read
+                     (concat "Tag: "
+                             (when emacs-wiki-tag-history
+                               (concat "(default: "
+                                       (car emacs-wiki-tag-history)
+                                       ") ")))
+                     (mapcar 'list
+                             (nconc (mapcar 'car emacs-wiki-markup-tags)
+                                    (mapcar 'car emacs-wiki-dangerous-tags)
+                                    emacs-wiki-custom-tags))
+                     nil nil nil
+                     'emacs-wiki-tag-history
+                     (car emacs-wiki-tag-history)))
+        tag-entry
+        (markup-opt ""))
+    (when (equal markup-tag "")
+      (setq markup-tag (car emacs-wiki-tag-history)))
+    ;; Get full tag entry
+    (setq tag-entry (or (assoc markup-tag emacs-wiki-markup-tags)
+                        (assoc markup-tag emacs-wiki-dangerous-tags)))
+    ;; Add to custom list if no entry exists
+    (unless tag-entry
+      (add-to-list 'emacs-wiki-custom-tags markup-tag))
+    ;; Get option
+    (when (nth 2 tag-entry)
+      (setq markup-opt (read-string "Option: ")))
+    (unless (equal markup-opt "")
+      (setq markup-opt (concat " " markup-opt)))
+    ;; Insert the tag, closing if necessary
+    (when markup-tag
+      (insert (concat "<" markup-tag markup-opt ">")))
+    (when (nth 1 tag-entry)
+      (insert (concat "\n\n</" markup-tag ">\n"))
+      (forward-line -2))))
+
+;; Load standard modules
+(require 'emacs-wiki-colors)
+(require 'emacs-wiki-macros)
+(require 'emacs-wiki-project)
 ;;; emacs-wiki.el ends here
